# Makefile.PL for adduser.pl package
#
# $Id: Makefile.PL,v 0.2 2003/12/15 21:24:28 acorliss Exp acorliss $
#

use Config;
use CPAN;
require 5.6.0;
use strict;

my @bdirs = qw(/bin /sbin /usr/bin /usr/sbin /usr/ccs/bin /usr/local/bin
  /usr/local/sbin);
my %binaries = (
  useradd       => undef,
  chmod         => undef,
  edquota       => undef,
  make          => undef,
  grep          => undef,
  perl          => undef,
  false         => undef,
  );
my %modules = (
  'CGI'                   => 0,
  'CGI::Carp'             => 0,
  'Parse::PlainConfig'    => 1,
  'Template'              => 2,
  );
my ($b, $dir, $fp, $confirmed, $line, $method, $uid, $dest);
my ($config, $v, @inst);

# Print the intro
print << '__EOF__';

adduser.pl -- CGI user creation interface.

The following sequence will attempt to configure this package correctly for
your working environment.

Phase 1:  Location of external binaries

__EOF__

# Check for the location of all applicable binaries
foreach  $b (keys %binaries) {
  $fp = "";
  $confirmed = 0;
  foreach $dir (@bdirs) {
    $fp = "$dir/$b" and last if -x "$dir/$b";
  }

  until ($confirmed) {
    print "Location of $b [$fp]? ";
    chomp($line = <STDIN>);
    $line = $fp if length($line) < 1;
    $confirmed = 1 if -x $fp;
    if ($b eq "edquota" && length($line) < 1) {
      print "Okay, but the quota system will be disabled. . .\n";
      $confirmed = 1;
    }
  }
  $binaries{$b} = $line;
}

# Check what kind of setuid execution can be used
print "\nPhase 2:  Suidperl/C wrapper\n\n";

# Check for suidperl
$fp = $binaries{perl};
$fp =~ s/perl/suidperl/;
if (-x $fp) {
  print "Hmmm, you seem to have suidperl present ($fp).\n";
  $method = "suidperl";
} else {
  print "Hmmm, you don't seem to have suidperl on this system, building\n",
    "a setuid C wrapper, instead.\n";
  $method = "c";
}

# Ask for the running UID to use
print "\nPhase 3:  UID allowed access\n\n";
until (defined $uid) {
  print "What UID can call this script (httpd server uid)? ";
  chomp($line = <STDIN>);
  next unless length($line) > 0;
  $fp = "cat /etc/passwd | awk -F: '{ if (\$3 == $line) { exit 5 }; }'";
  if (system($fp) / 256 == 5) {
    chomp($fp = 
      `cat /etc/passwd | awk -F: '{ if (\$3 == $line) { print \$1 }; }'`);
    print "Okay, using account for $fp.\n";
    $uid = $line;
  } else {
    print "I'm sorry, but I don't see such an account in /etc/password.\n";
  }
}

# Get the location for installation
print "\nPhase 4:  Installation location\n\n";

until (defined $dest) {
  print "Location to install script (site cgi-bin)? ";
  chomp($line = <STDIN>);
  if (-d $line) {
    print "Okay, using $line.\n";
    $dest = $line;
  } else {
    print "I'm sorry, but that directory doesn't appear to exist.\n";
  }
}

# Get the location for the conf file
print "\nPhase 5:  Conf-file location\n\n";

$fp = $dest;
$fp =~ s#[^/]+/?$#etc/adduser.conf#;
until (defined $config) {
  print "Location of the conf file [$fp]? ";
  chomp($line = <STDIN>);
  $line = $fp if length($line) < 1;
  if (-d ($line =~ m#^(.+)/[^/]+$#)[0]) {
    print "Okay, using $line.\n";
    $config = $line;
  } else {
    print "I'm sorry, but $1 doesn't appear to exist.\n";
  }
}

# Create the sed script
$binaries{perl} =~ s/perl/suidperl/ if $method eq "suidperl";
@binaries{qw(uid config)} = ($uid, $config);
open(SED, "> config.sed") || die "Couldn't open config.sed: $!\n";
foreach (keys %binaries) {
  print SED "s#__@{[uc($_)]}__#$binaries{$_}#\n";
}
close(SED);

# Make sure the requisite modules are installed
print "\nPhase 6:  Requisite modules\n\n";

foreach (sort keys %modules) {
  $b = eval("require $_");
  $v = $b != 0 ? $_->VERSION : 0;
  if ($b == 1 && $v > $modules{$_}) {
    print "Requisite module $_ present and meets minimum version.\n";
  } else {
    print "Requisite module $_ not present or too old.  Install newer one? ";
    chomp($line = <STDIN>);
    if ($line =~ /^y/i) {
      install $_;
    } else {
      print "Canceling installation.\n";
      exit 1;
    }
  }
}

# Compose the installation commands
@inst = (
  "sed -f ./config.sed adduser.tmpl > adduser.pl",
  "cp adduser.pl $dest/",
  "chmod 4755 $dest/adduser.pl",
  );
push(@inst, 
  "mv $dest/adduser.pl $dest/int-adduser.pl",
  "cat wrapper.tmpl | sed 's#NEWPATH#$dest#' > wrapper.c",
  "$Config{cc} -o adduser.pl wrapper.c",
  "strip --strip-debug adduser.pl",
  "mv adduser.pl $dest/",
  "chmod 4755 $dest/adduser.pl") if $method eq "c";

# Execute the installation commands
print "\nPhase 7: Installation\n\n";

foreach (@inst) {
  print "$_\n";
  unless (system($_) == 0) {
    print "Installation incomplete!  Please rectify errors above.\n\n";
    exit 1;
  }
}

print << "__EOF__";

Installation is complete!

It is assumed that you've already read the README and INSTALL file and done
everything you need to prep the system for this script.  If you haven't done
so already then READ THEM NOW!!!!!

I will also heavily encourage you to review the code in this script.  This is
a very dangerous feature to implement.  Please send any fixes, requests, or
comments to Arthur Corliss <corliss\@digitalmages.com>.

__EOF__

exit 0;

